package com.btcpiyush.ads.google_applovin_unity_ads

import android.app.Activity
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.util.Log
import androidx.annotation.NonNull
import com.applovin.mediation.*
import com.applovin.sdk.AppLovinMediationProvider
import com.applovin.sdk.AppLovinSdk
import com.applovin.sdk.AppLovinSdkSettings
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppLovinSetting
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppSettings
import com.btcpiyush.ads.google_applovin_unity_ads.banner_ad.AdMobBanner
import com.btcpiyush.ads.google_applovin_unity_ads.banner_ad.BannerAdViewFactory
import com.btcpiyush.ads.google_applovin_unity_ads.interestial_ad.AdmobInterstitial
import com.btcpiyush.ads.google_applovin_unity_ads.native_ad.AdMobNative
import com.btcpiyush.ads.google_applovin_unity_ads.native_ad.NativeAdViewFactory
import com.btcpiyush.ads.google_applovin_unity_ads.open_ad.AdmobOpenAd
import com.btcpiyush.ads.google_applovin_unity_ads.open_ad.AppStateNotifier
import com.btcpiyush.ads.google_applovin_unity_ads.reward_ad.AdmobReward
import com.facebook.ads.*
import com.google.android.gms.ads.*
import com.google.gson.Gson
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.platform.PlatformViewRegistry
import java.util.*

class GoogleApplovinUnityAdsPlugin : FlutterPlugin, ActivityAware, MethodCallHandler {

    companion object {
        val handler = Handler(Looper.getMainLooper())
        const val TAG = "GoogleApplovinPlugin"
        const val CHANNEL_NAME = "dev.bitcoding.ads/google_applovin_unity_ads"
        const val CHANNEL_BANNER_NAME = "dev.bitcoding.ads/banner"
        const val CHANNEL_INTERESTIAL_NAME = "dev.bitcoding.ads/interstitial"
        const val CHANNEL_OPENAD_NAME = "dev.bitcoding.ads/open_ad"
        const val CHANNEL_REWARD_NAME = "dev.bitcoding.ads/reward"
        const val CHANNEL_NATIVE_NAME = "dev.bitcoding.ads/native"
        const val METHOD_ADS_API_SETTING = "call.ads.api.setting"
        const val METHOD_LOAD_BANNER_ADS = "call.ads.banner"
        const val METHOD_LOAD_NATIVE_ADS = "call.ads.native"
        const val METHOD_SUCCESS = "SUCCESS"
        const val METHOD_ERROR = "ERROR"
        const val METHOD_DISPOSE_BANNER_ADS = "dispose.ads.banner"
        const val METHOD_DISPOSE_NATIVE_ADS = "dispose.ads.native"
        const val METHOD_BANNER_ADS = "ads.banner"
        const val METHOD_NATIVE_ADS = "ads.native"
        var interestialAdInterval: Long = 0L
        var appLovinSDK: AppLovinSdk? = null
    }

    private var applicationContext: Context? = null

    private var methodChannel: MethodChannel? = null
    private var interstitialChannel: MethodChannel? = null
    private var rewardChannel: MethodChannel? = null
    private var bannerChannel: MethodChannel? = null
    private var nativeChannel: MethodChannel? = null
    private var openAdChannel: MethodChannel? = null
    private lateinit var messenger: BinaryMessenger
    private var activity: Activity? = null
    private var mInterstitialAdIsLoading: Boolean = false
    private var mRewardAdIsLoading: Boolean = false
    private var progressBar: ProgressDialog? = null
    private lateinit var platformViewRegistry: PlatformViewRegistry
    private var settingResponse: AppSettings? = null
    private var appStateNotifier: AppStateNotifier? = null
    private lateinit var pluginBinding: FlutterPlugin.FlutterPluginBinding

    override fun onAttachedToEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        pluginBinding = binding
        applicationContext = binding.applicationContext
        platformViewRegistry = binding.platformViewRegistry
        messenger = binding.binaryMessenger
        methodChannel = MethodChannel(messenger, CHANNEL_NAME)
        interstitialChannel = MethodChannel(binding.binaryMessenger, CHANNEL_INTERESTIAL_NAME)
        rewardChannel = MethodChannel(binding.binaryMessenger, CHANNEL_REWARD_NAME)
        openAdChannel = MethodChannel(binding.binaryMessenger, CHANNEL_OPENAD_NAME)
        bannerChannel = MethodChannel(binding.binaryMessenger, CHANNEL_BANNER_NAME)
        nativeChannel = MethodChannel(binding.binaryMessenger, CHANNEL_NATIVE_NAME)
        appStateNotifier = AppStateNotifier(binding.binaryMessenger)
        methodChannel?.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        when (call.method) {
            METHOD_ADS_API_SETTING -> {
                loadSettingAPI(result, call.argument<String>("json_data").toString())
            }

            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel?.setMethodCallHandler(null)
        interstitialChannel?.setMethodCallHandler(null)
        rewardChannel?.setMethodCallHandler(null)
        openAdChannel?.setMethodCallHandler(null)
        if (appStateNotifier != null) {
            appStateNotifier?.stop()
            appStateNotifier = null
        }
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {}

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivity() {}

    private fun loadSettingAPI(result: MethodChannel.Result?, json_data: String) {
        try {
            settingResponse = Gson().fromJson(json_data, AppSettings::class.java)
            settingResponse?.let {
                activity?.let { act ->
                    progressBar = ProgressDialog(
                        act,
                        it.adSetting?.adsLoadingText ?: "Ads will loading..."
                    )
                    if ((it.adSetting?.appVersionCode ?: 1).toLong() == getVersionCode()) {
                        it.adsSequence.clear()
                    }
                    loadSdk(act, result)
                }
            }

        } catch (exp: Exception) {
        }
    }

    private fun getVersionCode() : Long {
        activity?.let { context ->
            try {
                val pInfo = context.packageManager.getPackageInfo(context.packageName, 0)
                return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                    pInfo.longVersionCode
                } else {
                    pInfo.versionCode.toLong()
                }
            } catch (e: PackageManager.NameNotFoundException) {
                e.printStackTrace()
            }
        }

        return 1L
    }

    private fun findMatch(s: String, strings: ArrayList<String>): Boolean {
        return strings.any { s.contains(it) }
    }

    private fun loadSdk(act: Activity, result: MethodChannel.Result? = null) {
        settingResponse?.let {
            interstitialChannel?.setMethodCallHandler(AdmobInterstitial(pluginBinding, act, it))
            rewardChannel?.setMethodCallHandler(AdmobReward(pluginBinding, act, it))
            openAdChannel?.setMethodCallHandler(AdmobOpenAd(pluginBinding, act, it))
            bannerChannel?.setMethodCallHandler(AdMobBanner(pluginBinding, act, it))
            nativeChannel?.setMethodCallHandler(AdMobNative(pluginBinding, act, it))
            if (it.adsSequence.size > 0) {
                var isBannerAds: Boolean = it.customAds?.isBannerAds == true
                var isNativeAds: Boolean = it.customAds?.isNativeAds == true

                if (findMatch(act.getString(R.string.google_ads).lowercase(), it.adsSequence)) {
                    it.googleAds?.let {
                        if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isOpenAds == true || it.isRewardedAds == true) {
                            isBannerAds = it.isBannerAds == true || isBannerAds
                            isNativeAds = it.isNativeAds == true || isNativeAds
                            MobileAds.initialize(act) {}

                        }
                    }
                }

                if (findMatch(act.getString(R.string.app_lovin).lowercase(), it.adsSequence)) {
                    it.appLovin?.let {
                        if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
                            isBannerAds = it.isBannerAds == true || isBannerAds
                            isNativeAds = it.isNativeAds == true || isNativeAds
                            _applovinSDKInitialize(act, it)
                        }
                    }
                }

                if (findMatch(act.getString(R.string.facebook_ads).lowercase(), it.adsSequence)) {
                    it.facebookAds?.let {
                        if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
                            isBannerAds = it.isBannerAds == true || isBannerAds
                            isNativeAds = it.isNativeAds == true || isNativeAds

                            if (BuildConfig.DEBUG) {
                                AdSettings.turnOnSDKDebugger(act)
                                AdSettings.isTestMode(act)
                                AdSettings.addTestDevice("55ebd8be-199d-4bdc-85eb-69e6e7fa6d66")
                            }

                            AudienceNetworkAds.initialize(act)
                        }
                    }
                }

                if (isBannerAds) {
                    platformViewRegistry.registerViewFactory(
                        METHOD_BANNER_ADS,
                        BannerAdViewFactory(act, it)
                    )
                }

                if (isNativeAds) {
                    platformViewRegistry?.registerViewFactory(
                        METHOD_NATIVE_ADS,
                        NativeAdViewFactory(act, it)
                    )
                }
            }
            sendSuccess(result, METHOD_SUCCESS)
        } ?: kotlin.run {
            sendSuccess(result, METHOD_ERROR)
        }
    }

    private fun loadSdk(act: Activity, result: MethodChannel.Result? = null, pos: Int = 0) {
        settingResponse?.let {
            interstitialChannel?.setMethodCallHandler(AdmobInterstitial(pluginBinding, act, it))
            rewardChannel?.setMethodCallHandler(AdmobReward(pluginBinding, act, it))
            openAdChannel?.setMethodCallHandler(AdmobOpenAd(pluginBinding, act, it))
            bannerChannel?.setMethodCallHandler(AdMobBanner(pluginBinding, act, it))
            nativeChannel?.setMethodCallHandler(AdMobNative(pluginBinding, act, it))
            if (it.adsSequence.size > 0) {
                if (it.adsSequence.size > pos) {
                    Log.e(TAG, "loadSdk: $pos   ${it.adsSequence[pos].lowercase()}")
                    when (it.adsSequence[pos].lowercase()) {
                        act.getString(R.string.google_ads).lowercase() -> {
                            initMobSdk(act, result, pos)
                        }
                        act.getString(R.string.facebook_ads).lowercase() -> {
                            initFacebookSdk(act, result, pos)
                        }
                        act.getString(R.string.app_lovin).lowercase() -> {
                            initAppLovinSDK(act, result, pos)
                        }
                        else -> {
                            loadSdk(act, result, pos + 1)
                        }
                    }
                } else {
                    Log.e(TAG, "loadSdk: $pos   Other Pos Data}")
                    val handler = Handler(Looper.getMainLooper())
                    handler.postDelayed({
                        mInterstitialAdIsLoading = false
                        mRewardAdIsLoading = false

                        progressBar?.dismiss()

                        if (it.googleAds?.isBannerAds == true ||
                            it.appLovin?.isBannerAds == true ||
                            it.facebookAds?.isBannerAds == true ||
                            it.customAds?.isBannerAds == true
                        ) {
                            platformViewRegistry.registerViewFactory(
                                METHOD_BANNER_ADS,
                                BannerAdViewFactory(act, it)
                            )
                        }

                        if (it.googleAds?.isNativeAds == true ||
                            it.appLovin?.isNativeAds == true ||
                            it.facebookAds?.isNativeAds == true ||
                            it.customAds?.isNativeAds == true
                        ) {
                            platformViewRegistry?.registerViewFactory(
                                METHOD_NATIVE_ADS,
                                NativeAdViewFactory(act, it)
                            )
                        }

                        sendSuccess(result, "SUCCESS")

                        // requestGoogleOpenAd(act)

                    }, 10)
                }
            } else {
                Log.e(TAG, "loadSdk: $pos   Other Pos Data}")
                sendSuccess(result, "SUCCESS")
            }
        }
    }

    private fun initMobSdk(act: Activity, result: MethodChannel.Result?, pos: Int) {
        try {
            settingResponse?.let { setting ->
                setting.googleAds?.let {
                    if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isOpenAds == true || it.isRewardedAds == true) {
                        MobileAds.initialize(act) {
                            loadSdk(act, result, pos + 1)
                        }
                        return@initMobSdk
                    }
                }
            }
        } catch (e: Exception) {
        }

        loadSdk(act, result, pos + 1)
    }

    private fun initFacebookSdk(act: Activity, result: MethodChannel.Result?, pos: Int) {
        try {
            settingResponse?.let {
                it.facebookAds?.let {
                    if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
                        if (BuildConfig.DEBUG) {
                            AdSettings.turnOnSDKDebugger(act)
                            AdSettings.isTestMode(act)
                            AdSettings.addTestDevice("55ebd8be-199d-4bdc-85eb-69e6e7fa6d66")
                        }
                        AudienceNetworkAds.initialize(act)
                    }
                }
            }
        } catch (e: Exception) {
        }
        loadSdk(act, result, pos + 1)
    }

    private fun initAppLovinSDK(act: Activity, result: MethodChannel.Result?, pos: Int) {
        settingResponse?.let {
            it.appLovin?.let {
                if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
//                    AppLovinSdk.getInstance(act).mediationProvider = "max"
//                    AppLovinSdk.initializeSdk(act)
                    _applovinSDKInitialize(act, it)
                }
            }
        }
        loadSdk(act, result, pos + 1)
    }

    private fun _applovinSDKInitialize(act: Activity, it: AppLovinSetting) {
        var sdkKeyToUse = it.appLovinSdkKey
        Log.e(TAG, "initAppLovinSDK: $sdkKeyToUse")
        if (TextUtils.isEmpty(sdkKeyToUse)) {
            try {
                applicationContext?.let {
                    val packageManager = it.packageManager
                    val packageName = it.packageName
                    val applicationInfo = packageManager.getApplicationInfo(
                        packageName,
                        PackageManager.GET_META_DATA
                    )
                    val metaData = applicationInfo.metaData
                    sdkKeyToUse = metaData.getString("applovin.sdk.key", "")
                    Log.e(TAG, "$sdkKeyToUse")
                }
            } catch (th: Throwable) {
                Log.e(TAG, "Unable to retrieve SDK key from Android Manifest: $th")
            }
            // check(!TextUtils.isEmpty(sdkKeyToUse)) { "Unable to initialize AppLovin SDK - no SDK key provided and not found in Android Manifest!" }
        }

        Log.e(TAG, "$sdkKeyToUse")

        try {// Initialize SDK
            appLovinSDK = AppLovinSdk.getInstance(
                sdkKeyToUse,
                AppLovinSdkSettings(act),
                act
            )
        } catch (e: Exception) {
            appLovinSDK = null
        }

        appLovinSDK?.mediationProvider = AppLovinMediationProvider.MAX
        appLovinSDK?.settings?.setVerboseLogging(true)

        appLovinSDK?.initializeSdk { configuration ->
            Log.e(TAG, "APPLOVIN SDK initialized")
        }
    }


    private fun getGoogleAdRequest(): AdRequest {
        return AdRequest.Builder().build()
    }

    private fun showLog(error: String) {
        Log.e(TAG, error)
    }

    private fun sendSuccess(result: MethodChannel.Result?, msg: String) {
        try {
            handler.postDelayed({
                methodChannel?.invokeMethod(msg, null)
                //  result?.success(msg)
            }, 10L)
        } catch (e: Exception) {
        }
    }
}
